@@ -36,6 +36,8 @@
 namespace QPulseAudio
 {
 
+class Server;
+
 class Q_DECL_EXPORT Context : public QObject
 {
     Q_OBJECT
@@ -45,6 +47,10 @@ public:
 
     static Context *instance();
 
+    static const qint64 NormalVolume = PA_VOLUME_NORM;
+    static const qint64 MinimalVolume = 0;
+    static const qint64 MaximalVolume = (PA_VOLUME_NORM / 100.0) * 150;
+
     void ref();
     void unref();
 
@@ -56,6 +62,8 @@ public:
     const SourceOutputMap &sourceOutputs() const { return m_sourceOutputs; }
     const ClientMap &clients() const { return m_clients; }
     const CardMap &cards() const { return m_cards; }
+    const StreamRestoreMap &streamRestores() const { return m_streamRestores; }
+    Server *server() const { return m_server; }
 
     void subscribeCallback(pa_context *context, pa_subscription_event_type_t type, uint32_t index);
     void contextStateCallback(pa_context *context);
@@ -66,24 +74,34 @@ public:
     void sourceOutputCallback(const pa_source_output_info *info);
     void clientCallback(const pa_client_info *info);
     void cardCallback(const pa_card_info *info);
+    void streamRestoreCallback(const pa_ext_stream_restore_info *info);
+    void serverCallback(const pa_server_info *info);
 
     void setCardProfile(quint32 index, const QString &profile);
+    void setDefaultSink(const QString &name);
+    void setDefaultSource(const QString &name);
+    void streamRestoreWrite(const pa_ext_stream_restore_info *info);
 
     template <typename PAFunction>
-    void setGenericVolume(quint32 index, qint64 newVolume,
+    void setGenericVolume(quint32 index, int channel, qint64 newVolume,
                           pa_cvolume cVolume, PAFunction pa_set_volume)
     {
-        // TODO: overdrive
-        newVolume = qBound<qint64>(0, newVolume, 65536);
+        newVolume = qBound<qint64>(0, newVolume, PA_VOLUME_MAX);
         pa_cvolume newCVolume = cVolume;
-        for (int i = 0; i < newCVolume.channels; ++i) {
-            newCVolume.values[i] = newVolume;
+        if (channel == -1) { // -1 all channels
+            for (int i = 0; i < newCVolume.channels; ++i) {
+                newCVolume.values[i] = newVolume;
+            }
+        } else {
+            Q_ASSERT(newCVolume.channels > channel);
+            newCVolume.values[channel] = newVolume;
         }
         if (!PAOperation(pa_set_volume(m_context, index, &newCVolume, nullptr, nullptr))) {
             qCWarning(PLASMAPA) <<  "pa_set_volume failed";
             return;
         }
     }
+
     template <typename PAFunction>
     void setGenericMute(quint32 index, bool mute, PAFunction pa_set_mute)
     {
@@ -122,7 +140,7 @@ public:
     }
 
 private:
-    Q_INVOKABLE void connectToDaemon();
+    void connectToDaemon();
     void reset();
 
     // Don't forget to add things to reset().
@@ -132,6 +150,8 @@ private:
     SourceOutputMap m_sourceOutputs;
     ClientMap m_clients;
     CardMap m_cards;
+    StreamRestoreMap m_streamRestores;
+    Server *m_server;
 
     pa_context *m_context;
     pa_glib_mainloop *m_mainloop;
